% set up the sizes

m = 10
n = 10

% create a vector mx1 full of zeros
x = zeros(m,1)
% create a vector nx1 full of ones
y = ones(n,1)

%% create a random vector (usefull shen testing algorithms)
x = rand(m,1)

%% access entries of x
x(1)

%% define an index
i = 5
% accees i-th element in x
x(i)

%% modify an entry of x
x(i) = 100

x

%% create a matrix (random)
A = rand(m,n)

%% create another index
j = 3

% access i,j index of A
A(i,j)
%% access a column
A(:,j)

%% access a row
A(i,:)

%% compute the inner product
% you can add a semicolon at the end so it doesn't show in the console

x = rand(m,1);
y = rand(m,1);

innerproduct = 0;
for i = 1:m
   innerproduct = innerproduct  + x(i)*y(i);
end

fprintf('Inner product is %d \n', innerproduct)

%% Row reduction 

A = [1 2 3; ...
     4 5 6; ...
     6 7 8] ;
 
 % we can transpose vectors 
 b = [ 4 7 9 ].'
 
 %obtainingthe size of A
 [m,n] = size(A)
 
 % computing augmented matrix
 
 B = [A b];
 
 for i = 2:m
     B(i,:) = B(i,:) - B(1,:)*B(i,1)
 end
 
 if (B(2,2) == 0 )
     fprintf('pivot equals to zero! either swap or finish \n ');
 end
 B(2,:) =  B(2,:)/B(2,2)
 
 for i = 3:m
     B(i,:) = B(i,:) - B(2,:)*B(i,2)
 end
 
 if (B(3,3) == 0 )
     fprintf('pivot equals to zero either swap or finish \n');
 end
 %% test for consistency
 
 for i = 1:m
     normrow = norm(B(i,1:n));
     if (normrow == 0) && (B(i,n+1)~= 0)
         fprintf('matrix is not consistent \n')
     end
 end

 
 
 
 %% Row reduction using vectorized operations
 
% If you don't want to deal with the for loops to knock out elements
% on your matrices you can perform vectorized operations 

% let consider a 5x5 matrix A

A = [ 1 2 3 4 5; ...
      1 1 2 2 5; ...
     -1 2 3 2 1; ...
      1 4 1 2 3; ...
      1 3 4 1 0]
% and a right hand side b

b = [1 1 1 7 -4].'

% obtainingthe size of A
[m,n] = size(A)

% following what you have learned in class, you form the 
% augmented matrix 

B = [A b];

%% we start by the first row
j = 1;
% we put a one in the pivot  
B(j,:) = B(j,:)/B(j,j)

% instead of performing the for loop we can use an vectorized operaior

B(j+1:m,:) = B(j+1:m,:) - B(j+1:m,j)*B(j,:)

% you can observe that you knocked-out all the entries below the first
% column.

%% now we perform the same for the second column
j = 2;
% we check that the pivot is different than zero
if (B(j,j) == 0 )
    fprintf('pivot equals to zero! either swap or finish \n ');
end

% it is not, so we can continue 
% we normalize the pivot
B(j,:) = B(j,:)/B(j,j)

% and we knock out the entried below the diagonal in the second column
B(j+1:m,:) = B(j+1:m,:) - B(j+1:m,j)*B(j,:)

%% now we perform the same for the third column
j = 3;
% we check that the pivot is different than zero
if (B(j,j) == 0 )
    fprintf('pivot equals to zero! either swap or finish \n ');
end

% it is not, so we can continue 
% we normalize the pivot
B(j,:) = B(j,:)/B(j,j)

% and we knock out the entried below the diagonal in the second column
B(j+1:m,:) = B(j+1:m,:) - B(j+1:m,j)*B(j,:)
 

%% now we perform the same for the fourth column
j = 4;
% we check that the pivot is different than zero
if (B(j,j) == 0 )
    fprintf('pivot equals to zero! either swap or finish \n ');
end

% it is not, so we can continue 
% we normalize the pivot
B(j,:) = B(j,:)/B(j,j)

% and we knock out the entries below the diagonal in the second column
B(j+1:m,:) = B(j+1:m,:) - B(j+1:m,j)*B(j,:)


%% now we perform the same for the last column :)
j = 5;
% we check that the pivot is different than zero
if (B(j,j) == 0 )
    fprintf('pivot equals to zero! either swap or finish \n ');
end

% it is not, so we can continue 
% given that this is the last column, we only need to normalize
B(j,:) = B(j,:)/B(j,j)

%% Reduction to Row reduced echelon form 
% The matrix is in echelon form, and in this case all the rows 
% have a pivot, and the system is consistent. 
% In this case we can go ahead and row reduce the matrix to 
% reduced echelon form

% The row reduction is the same for every column
% the only thing that changes is the row index i. 

% We can then row reduce B using just a for loop

% we go from the bottom to the top of the matrix
for j = n:-1:1
    B(j,:) = B(j,:)/B(j,j); % normalize 
    % we are eliminating entries above the diagonal
    % (check how is different from the one before)
    B(1:j-1,:) = B(1:j-1,:) - B(1:j-1,j)*B(j,:)
end

%% getting the solution

% now that the augmented matrix is in reduced echelon form you can extract
% the answer
x = B(:,n+1)

%and we test that it is the solution
A*x - b

